

puts "loading sys.S9418.tcl"

variable registerlist

lappend registerlist [list NetVar.S9418_NumSamples  "-n S9418_NumSamples -t int"]
lappend registerlist [list NetVar.S9418_VMERetries  "-n S9418_VMERetries -t int"]
lappend registerlist [list NetVar.S9418_BERREnable  "-n S9418_BERREnable -t int"]
lappend registerlist [list NetVar.S9418_VMEBufSize  "-n S9418_VMEBufSize -t int"]


proc Set_S9418_Options {} {puts "Set_S9418_Options"
    variable Options

    load_Options

   if {![info exists Options(S9418_MaxEvtsPerBuf)]}         {set Options(S9418_MaxEvtsPerBuf) 1}
   if {![info exists Options(S9418_MaxBLT)]}                {set Options(S9418_MaxBLT) 1}
   if {![info exists Options(S9418_MemBufSize)]}            {set Options(S9418_MemBufSize) 1024}
   if {![info exists Options(S9418_VMEBufSize)]}            {set Options(S9418_VMEBufSize) 16384}

   if {![info exists Options(S9418_HistEnable)]} {set Options(S9418_HistEnable) 1}
   if {![info exists Options(S9418_channels)]}   {set Options(S9418_channels) 4096}
   if {![info exists Options(S9418_offset)]}     {set Options(S9418_offset) 0}
   if {![info exists Options(S9418_shift)]}      {set Options(S9418_shift) 0}

   if {![info exists Options(S9418_GroupBase)]}  {set Options(S9418_GroupBase) 160}   ;#  default Group Base

    update_Options
}

proc S9418_Configure {} {puts "S9418_Configure"
    variable Options

    set Options(S9418_VMEBufSize) [ReadNetVar NetVar.S9418_VMEBufSize]

    update_Options
}

proc Dump_Raw_VME_Data_S9418 {data} {
     
     set info ""

     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 24 & 7]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 16 & 0x3f]
             set un [expr $word >> 13 & 1]
             set ov [expr $word >> 12 & 1]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i; un= %3i; ov= %3i; data= 0x%5x" $word $event_type $channel $un $ov $adc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word >> 8 & 0x3f]
             append info "[format "0x%8x    %8s   module= %5i; channels= %6i" $word $event_type $geo $counter]"
            }
            4 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0xffffff]
             append info "[format "0x%8x    %8s   module= %5i; event data counter= %10i" $word $event_type $geo $counter]"
            }
            default {
             append info "$word - not valid datum\n"
            }
         }
         append info "\n"
     }
   return $info
}


proc S9418_DAQ-go {} {puts "S9418_DAQ-go"

      WriteVMERegisters s9418#*.Ctrl_Reg1 0
}

proc S9418_DAQ-halt {} {puts "S9418_DAQ-halt"

      WriteVMERegisters s9418#*.Ctrl_Reg1 1
}

proc S9418_DAQ-setup {} {puts "S9418_DAQ-setup"

      S9418_Configure

    WriteVMERegisters s9418#*.Ctrl_Reg1 1

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr s9418#$i.VMEStatus} m]
      if {$z!=0} then {break}
      WriteVMERegisters s9418#$i.Vsn2 $i
      incr maxmodule
   }

}

proc S9418_DAQ-flush {} {puts "S9418_DAQ-flush"}

proc S9418_DAQ-reset {} {puts "S9418_DAQ-reset"

      WriteVMERegisters s9418#*.Ctrl_Reg1 1
}

proc define_S9418_config {} {puts "define_S9418_config"
   variable Options

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr s9418#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "9418 0 [format "0x%x" $Options(S9418_GroupBase)]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr s9418#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register S9418#$i.Output_Buffer"
          error $m
      }

      DefineConfig "9418 $i [string range [first $m] 2 end]"
   }
}


proc create_S9418_spectra {} {puts "create_S9418_spectra"
    variable Options 

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr s9418#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }
   set maxadc 32

   for {set i 1} {$i <= $maxmodule} {incr i} {

      for {set j 0} {$j < $maxadc} {incr j} {

         if {$Options(S9418_HistEnable) > 0} {

#    histograms for the ADC

           set spectrumname S9418#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(S9418_channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * $maxadc) + $Options(S9418_GroupBase)] 
              map_spectrum  "$spectrumname $group $j $Options(S9418_offset) $Options(S9418_shift)"
           }
         }
      }
   }
}



