

puts "loading sys.V1725.tcl"

variable registerlist

lappend registerlist [list NetVar.V1725_NumSamples  "-n V1725_NumSamples -t int"]
lappend registerlist [list NetVar.V1725_VMERetries  "-n V1725_VMERetries -t int"]
lappend registerlist [list NetVar.V1725_BERREnable  "-n V1725_BERREnable -t int"]
lappend registerlist [list NetVar.V1725_VMEBufSize  "-n V1725_VMEBufSize -t int"]

variable NumChans 16
variable V1725_Base 96

proc Set_V1725_Options {} {
    variable Options
    variable V1725_Base  

puts "Set_V1725_Options"

    load_Options

   if {![info exists Options(V1725_MaxBLT)]}                {set Options(V1725_MaxBLT) 1}
   if {![info exists Options(V1725_MemBufSize)]}            {set Options(V1725_MemBufSize) 8192}
   if {![info exists Options(V1725_VMEBufSize)]}            {set Options(V1725_VMEBufSize) 16384}

   if {![info exists Options(V1725_Hist_Wave.Enable)]}      {set Options(V1725_Hist_Wave.Enable) 0}
   if {![info exists Options(V1725_Hist_Wave.channels)]}    {set Options(V1725_Hist_Wave.channels) 8192}
   if {![info exists Options(V1725_Hist_Wave.offset)]}      {set Options(V1725_Hist_Wave.offset) 0}
   if {![info exists Options(V1725_Hist_Wave.shift)]}       {set Options(V1725_Hist_Wave.shift) 0}
 
   if {![info exists Options(V1725_GroupBase)]}             {set Options(V1725_GroupBase) $V1725_Base}
   set V1725_Base  $Options(V1725_GroupBase)

    update_Options
}

proc V1725_Configure {} {
    variable Options
    variable NumChans
    variable V1725_Base

      set z [catch {ReadVMERegister v1725#1.BoardInfo} m]
      if {$z==0} then {set NumChans [expr ($m >> 16) & 255]}

puts "V1725_Configure: channels=$NumChans; GroupBase=$V1725_Base"

    set VME_BUFFER_SIZE [ReadNetVar NetVar.V1725_VMEBufSize]
    set Options(V1725_VMEBufSize) $VME_BUFFER_SIZE

      update_Options

           switch $Options(V1725_MemBufSize) {
              65536 {set BUFF_ORG 2; set BUFF_NUM 4}
              32768 {set BUFF_ORG 3; set BUFF_NUM 8}
              16384 {set BUFF_ORG 4; set BUFF_NUM 16}
              8192  {set BUFF_ORG 5; set BUFF_NUM 32}
              4096  {set BUFF_ORG 6; set BUFF_NUM 64}
              2048  {set BUFF_ORG 7; set BUFF_NUM 128}
              1024  {set BUFF_ORG 8; set BUFF_NUM 256}
              512   {set BUFF_ORG 9; set BUFF_NUM 512}
              256   {set BUFF_ORG 10; set BUFF_NUM 1024}
              default {set BUFF_ORG 8; set BUFF_NUM 256}
           }

      set Buf_Size [expr ($Options(V1725_MemBufSize) / $BUFF_NUM)]  ;# buffer in 32 bit words

      set z [catch {ReadVMERegisterAttr v1725#1.VMEStatus} m]
      if {$z==0} then {

           set z [catch {WriteVMERegister v1725#1.VMEControl 16} m];  #  force BERR option

           set z [catch {ReadVMERegister v1725#1.CustomSize} SIZE]
           if {$SIZE <= 0} {set SIZE 32}
           WriteNetVar NetVar.V1725_NumSamples [expr $SIZE*10]

           set WORDS [expr ($SIZE * 10) / 2]   ;# convert to 32 bit words
           set NUM 1                           ;# events per buffer

           set z [catch {ReadVMERegister v1725#1.VMEControl} BERR]  ;#  enable BERR to finish a VME block transfer
           if {$BERR < 0} {set BERR 0}
           WriteNetVar NetVar.V1725_BERREnable [expr ($BERR >> 4) & 1]

puts "V1725 Output buffers $BUFF_NUM  @ $Buf_Size 32 bit words"
puts "Waveform [expr $SIZE * 10] samples"
puts "$NumChans channels; $NUM events/buffer"

           set osize [expr ($WORDS * $NUM) + 2]
puts "V1725 buffer size used $osize words out of max $Options(V1725_MemBufSize)"

           if {$osize > $Options(V1725_MemBufSize)} {
              puts "Number of samples or Events per buffer is too big\n\
                 $osize > $Options(V1725_MemBufSize) - this is not permitted\n\
                 correct options and try again"
              return
           }

           set BLT [expr $VME_BUFFER_SIZE / (($osize * $NumChans) + 4)]
           if {$BLT > $Options(V1725_MaxBLT)} {set BLT $Options(V1725_MaxBLT)}

puts "BLT Event Number = $BLT"
           set isize [expr (($osize * $NumChans) + 4) * $BLT]
puts "Data Acq Input buffer size used $isize words out of max $VME_BUFFER_SIZE"

puts "V1725_Configure $BERR $SIZE $BUFF_ORG $NUM"

           WriteVMERegisters v1725#*.VMEControl $BERR     ;# set all modules to same setting
           WriteVMERegisters v1725#*.RecLen $SIZE         ;# set all modules to same setting
           WriteVMERegisters v1725#*.BufferOrg $BUFF_ORG
           WriteVMERegisters v1725#*.BLTEventNum $BLT     ;# set all modules to same setting
      }

puts "V1725_Configure finished"

}

proc V1725_DAQ-go {} {puts "V1725_DAQ-go"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }
   if {$maxmodule == 0} {return}

   for {set i 1} {$i <= $maxmodule} {incr i} {    ;#   clear data buffers 
      set z [catch {WriteVMERegister v1725#$i.Clear 0} m]
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {    ;#  phase 1 - ARM if starting by external signal
      set z [catch {ReadVMERegister v1725#$i.AcqControl} m]
      if {$z == 0} {
         if {[expr ($m & 3)] != 0} {
             WriteVMERegister v1725#$i.AcqControl [expr ($m & 0xfffb) + 4]
         }
      }
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {    ;#  phase 2 - START if using software control
      set z [catch {ReadVMERegister v1725#$i.AcqControl} m]
      if {$z == 0} {
         if {[expr ($m & 3)] == 0} {
             WriteVMERegister v1725#$i.AcqControl [expr ($m & 0xfffb) + 4]
         }
      }
   }
}

proc V1725_DAQ-halt {} {puts "V1725_DAQ-halt"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }
   if {$maxmodule == 0} {return}

   for {set i 1} {$i <= $maxmodule} {incr i} {    ;#  STOP or DISARM

      set z [catch {ReadVMERegister v1725#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1725#$i.AcqControl [expr ($m & 0xfffb)]}
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {    ;#   clear data buffers 
      set z [catch {WriteVMERegister v1725#$i.Clear 0} m]
   }

}

proc V1725_DAQ-setup {} {puts "V1725_DAQ-setup"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }
   if {$maxmodule == 0} {return}

   V1725_Configure
}

proc V1725_DAQ-flush {} {puts "V1725_DAQ-flush"

   for {set i 1} {$i <= $maxmodule} {incr i} {    ;#   clear data buffers 
      set z [catch {WriteVMERegister v1725#$i.Clear 0} m]
   }
}

proc V1725_DAQ-reset {} {puts "V1725_DAQ-reset"

   V1725_DAQ-halt
}

proc Dump_Raw_VME_Data_V1725 {data} {

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"

   return $info
}

proc define_V1725_config {} {puts "define_V1725_config"
    variable Options
    variable V1725_Base

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "17250 0 [format "0x%x" $V1725_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v1725#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v1725#$i.Output_Buffer"
          error $m
      }

      DefineConfig "17250 $i [string range [first $m] 2 end]"
   }
}

proc create_V1725_spectra {} {
    variable V1725_Base
    variable Options 
    variable NumChans

puts "create_V1725_spectra"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1725_Hist_Wave.Enable) == 1} {

        for {set j 0} {$j < $NumChans} {incr j} {   

#    histograms for the waveform

           set spectrumname v1725#$i.W[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1725_Hist_Wave.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * $NumChans) + $j  + $V1725_Base + 256] 
              set item 0
              map_spectrum  "$spectrumname $group $item $Options(V1725_Hist_Wave.offset) $Options(V1725_Hist_Wave.shift)"
           }
        }
     }

   }
}

