

#    experiment run control for CAEN - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

variable Options

variable SpecBase [file join $env(MIDASBASE) SpecData CAEN]
variable SpecPath


set z [catch {variable Modules $::VMEConfig::ModuleMenu} m]
if {$z != 0} {variable Modules [list CAEN_V1720 CAEN_V1724 CAEN_V1724PHA CAEN_V1725 CAEN_V1725PHA CAEN_V1725PSD CAEN_V1730 CAEN_V1730PHA CAEN_V1730PSD CAEN_V1740 CAEN_V1751 CAEN_V1751PSD CAEN_V1495DPP CAEN_V1495 CAEN_V1495V2]}

variable Supported_Modules {}
foreach item $Modules {
     if {$item == "S9418"} {lappend Supported_Modules $item; continue}
     if {$item == "SAC"} {lappend Supported_Modules $item; continue}
     if {$item == "PatternGen"} {lappend Supported_Modules $item; continue}
     if {$item == "VData"} {lappend Supported_Modules $item; continue}
     if {$item == "MyRIAD"} {lappend Supported_Modules $item; continue}
     set m [second [split $item _]]
     lappend Supported_Modules $m
}

puts "ConfigBase = $::VMEConfig::ConfigBase"
puts "Supported_Modules = $Supported_Modules"



   proc Set_Options {} {
      variable Options
      variable Supported_Modules

      LoadOptions

      if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 512}
      if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 512}
      if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
      if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 0}

   if {![info exists Options(VME.settings)]} {set Options(VME.settings) undefined}
   if {![info exists Options(Usepatterngen)]} {set Options(Usepatterngen) 0}

   if {![info exists Options(UseSAC)]} {set Options(UseSAC) 0}
   if {![info exists Options(Use1495)]} {set Options(Use1495) 0}
   if {![info exists Options(Use1495V2)]} {set Options(Use1495V2) 0}
   if {![info exists Options(Use1495DPP)]} {set Options(Use1495DPP) 0}

      foreach module $Supported_Modules {
         set Options(Include.$module)  1
         set Options(Histogram.$module) 1
      }

      SaveOptions
   }

   proc Dump_Raw_Data {} {
      variable Supported_Modules

#      foreach module $Supported_Modules {append info [Dump_Raw_[set module]_Data]}

    set info "Raw data dump for CAEN VME modules\n"

      set maxmodule 0
      foreach module $Supported_Modules {
         for {set i 1} {$i <= 16} {incr i} {
            set z [catch {ReadVMERegisterAttr [string tolower $module]#$i.VMEStatus} m]
            if {$z!=0} then {break}
            incr maxmodule
         }
      }

    for {set i 1} {$i <= $maxmodule} {incr i} {

       append info "VXXXX#$i\n"

        DataAcqCommand 19 "Getting Raw Data" "1 $i" 2
        set data [ReadNetVar NetVar.EXEC.Response]

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"
     }


       append info "VME\n"

        DataAcqCommand 19 "Getting Raw Data" "5 0" 2
        set data [ReadNetVar NetVar.EXEC.Response]

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"

     return $info

   }

proc doGo {} {

puts "executing CAEN doGo"

     Set_Options; make-UIregisters

     if {[ReadNetVar NetVar.DACQ.Ctxenable] == 1}  {
         DataAcqCommand 3 "connect xfer stream #0" 0 2
     }

     if {[ReadNetVar NetVar.DACQ.Ctx2enable] == 1}  {
         DataAcqCommand 6 "connect xfer stream #1" 0 2
     }

     Reset_Electronics

     DataAcqCommand 2 "Go"

     Go_Electronics

puts "completed CAEN doGo"

}

   proc Stop_Electronics {} {
       variable Options 
       variable Supported_Modules

       if {$Options(Usepatterngen) == 1} {
          WriteVMERegister patterngen#1.Start  0
       }

       if {$Options(UseSAC) != 0} {WriteVMERegister sac#1.STR 0}   ;#  disable acquisition
       if {$Options(Use1495) != 0} {WriteVMERegister v1495#1.Config.B8 0}     ;#  disable acquisition
       if {$Options(Use1495V2) != 0} {}     ;#  disable acquisition
       if {$Options(Use1495DPP) != 0} {WriteVMERegister v1495dpp#1.FT_RUN 0}  ;# disable acquisition

       foreach m $Supported_Modules {[set m]_DAQ-halt}

     after 2000   ;#  allow DA program to flush data 

   }

   proc Go_Electronics {} {
      variable Options 
      variable Supported_Modules

puts "starting Go_Electronics"

puts "Usepatterngen=$Options(Usepatterngen)"
puts "UseSAC=$Options(UseSAC)"
puts "Use1495=$Options(Use1495)"
puts "Use1495V2=$Options(Use1495V2)"
puts "Use1495DPP=$Options(Use1495DPP)"

       if {$Options(Usepatterngen) == 1} {
           set z [catch {WriteVMERegister patterngen#1.Start  3} m]
       }

       foreach m $Supported_Modules {[set m]_DAQ-go}

       if {$Options(UseSAC) != 0} {WriteVMERegister sac#1.STR 9;  WriteVMERegisters sac#1.ARM 0}
       if {$Options(Use1495DPP) != 0} {WriteVMERegister v1495dpp#1.FT_RUN 1}  ;# enable acquisition
       if {$Options(Use1495) != 0} {WriteVMERegister v1495#1.Config.B8 1}  ;# enable acquisition
       if {$Options(Use1495V2) != 0} {}  ;# enable acquisition

puts "Go_Electronics complete"
   }
 
   proc Reset_Electronics {} {    ;#   used by RESET
      variable Supported_Modules

      foreach m $Supported_Modules {[set m]_DAQ-reset}
   }

   proc Setup_Electronics {} {    ;#   used by SETUP
       variable Supported_Modules
       variable Options

       define_VME_addresses   ;#  setup VME 

       foreach m $Supported_Modules {[set m]_DAQ-setup}

   }

   proc define_VME_addresses {} {
       variable Supported_Modules
       variable Options

#    load hardware configuration into readout software

       ClearConfig
 
       foreach module $Supported_Modules {
          if {$Options(Include.$module) == 1} then {define_[set module]_config}
       }

       DataAcqCommand 4 "print config" 0 2

   }

    proc TerminateExec {} {

#        unmap_spectra
#        DataAcqCommand 22 "Terminating DataAcq program" 0 2

       define_VME_addresses

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin Linux64 CAENExecV8] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }


    foreach module $Supported_Modules {

        set f [file join $env(MIDASBASE) TclHttpd Html CAEN RunControl sys.$module.tcl]
        set z [catch {source $f} m]
        if {$z == 0} {puts "RunControl loaded $f"} else {puts "RunControl returned z=$z and $m (loading $f failed)"}
    }

    make-UIregisters

    Set_Options

    foreach module $Supported_Modules {Set_[set module]_Options}







