
#    for this module "channel" refers to a group of 8 channels
#    as with other modules in this family there are 8 "channels" or groups and hence 64 real inputs



# variable started 0

if {![info exists started]} {variable started 0}

if {$started != 0} {

     variable JS ""
     variable LogFlag

     if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}

    do_click
    UpdateModule
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1

    variable LogText ""
    variable LogFlag 0
    variable LogCount 0
    variable DebugFlag 0

    variable ShowLogFlag 0

    variable JS ""

    variable allmodules 0
    variable allchannels 0

    variable ModuleName v1740
    variable ModuleMenu {}
    variable ModuleAddress {}
    variable ModuleIndex 1

    variable SoftTrigRun 0


#    for the Register Menus each item is a list
#      0) name for menu
#      1) register
#      2) file containing support implementation
#      3) Display procedure  (null if none)
#      4) parameter update procedure   (null if none)
#      5) parameter save procedure (null if none)
#      6) parameter restore procedure (null if none)

    variable ModuleRegMenu { \
       {"Configuration"                       Config        Config.tml        GetConfig DrawConfig null null} \
       {"Buffer Organisation"                 BufferOrg     BuffOrg.tml       GetBuffOrg DrawBuffOrg null null} \
       {"Acquisition Window"                  CustomSize    CustSize.tml      GetCustSize DrawCustSize null null} \
       {"Group Enable Mask"                   Enable        GrpEnable.tml      GetGrpEnable DrawGrpEnable null null} \
       {"Trigger Source Enable Mask"          TrigSrcEnable TrigSrcEnable.tml GetTrigSrcEnable DrawTrigSrcEnable null null} \
       {"Front Panel Trigger Out Enable Mask" FPTrigOutEnable  FPTrigEnable.tml  GetFPTrigEnable DrawFPTrigEnable null null} \
       {"Front Panel I/O Data"                FPIOData      FPIOData.tml      GetFPIOData DrawFPIOData null null} \
       {"Front Panel I/O Control"             FPIOControl   FPIOCtrl.tml      GetFPIO DrawFPIO null null} \
       {"Post Trigger Setting"                PostTrig      PostTrig.tml      GetPostTrig DrawPostTrig null null} \
       {"Monitor DAC"                    MonitorDAC    MonitorDAC.tml  GetMonitorDAC DrawMonitorDAC null null} \
       {"Monitor Mode"                   MonitorMode   MonitorMode.tml GetMonitorMode DrawMonitorMode null null} \
       {"Acquisition Control"                 AcqControl    AcqCtrl.tml       GetAcqCtrl DrawAcqCtrl null null} \
       {"Acquisition Status"                  AcqStatus     AcqStatus.tml     GetAcqStatus DrawAcqStatus null null} \
       {"VME Control"                         VMECtrl       VMECtrl.tml       GetVMECtrl DrawVMECtrl null null} \
       {"VME Status"                          VMEStatus     VMEStatus.tml     GetVMEStatus DrawVMEStatus null null} \
       {"System Options"                      SystemOpts    SystemOpts.tml    GetSystemOpts DrawSystemOpts null null} \
    }

    variable ChRegMenu { \
       {"Status"                Status   GrpStatus.tml  GetGrpStatus DrawGrpStatus null null} \
       {"DAC"                   DAC      DAC.tml        GetDAC DrawDAC SaveDAC RestDAC} \
       {"Threshold"             Threshold Threshold.tml GetThreshold DrawThreshold SaveThreshold RestThreshold} \
       {"Channel Trigger Enable Mask"   ChEnable  ChEnable.tml  GetChEnable DrawChEnable SaveChEnable RestChEnable} \
    }

#   registers for Save/Restore not handled by above (module registers)
    variable SaveRestoreMenu {Enable Config BufferOrg TrigSrcEnable FPTrigOutEnable FPIOControl \
                 AcqControl CustomSize PostTrig}


    variable ModuleRegItem [lindex $ModuleRegMenu 0]
    variable ModuleReg [lindex $ModuleRegItem 0]

    variable ChRegItem [lindex $ChRegMenu 0]
    variable ChReg [lindex $ChRegItem 0]


#    source all implementation files to load implementation code
  foreach item $ModuleRegMenu  {
     source [file join V1740 [lindex $item 2]]
     set updateproc [lindex $item 3]
     if {$updateproc != "null"} {eval $updateproc}
  }

  foreach item $ChRegMenu  {
     source [file join V1740 [lindex $item 2]]
     set updateproc [lindex $item 3]
     if {$updateproc != "null"} {eval $updateproc}
  }



   proc do_RegisterHelp {W} {
      variable [set W]RegItem

       set f [lindex [set [set W]RegItem] 2]

        set Txtfile [file join V1740 [file rootname $f].txt]
        set z [catch {open $Txtfile r} file]
        if {$z != 0} {
            Alert "Unable to access $Txtfile\nInformation coming very soon!!\nFor now read the manual!!"
            return
        }

        set text [read $file]
        close $file

        Alert "$text"
  }

   proc do_ExpertHelp {} {

        set Txtfile [file join V1740 Expert.txt]
        set z [catch {open $Txtfile r} file]
        if {$z != 0} {
            Alert "Unable to access $Txtfile\nInformation coming very soon!!\nFor now read the manual!!"
            return
        }

        set text [read $file]
        close $file

        Alert "$text"
  }

   proc show_variables {} {
        variable allchannels

        variable allmodules
        variable LogFlag
        variable DebugFlag
        variable ShowLogFlag
        variable ModuleRegItem
        variable ChRegItem
        variable Rmsg
        variable Wmsg
        variable SoftTrigRun

             InsertLog "LogFlag=$LogFlag; ShowLogFlag=$ShowLogFlag; DebugFlag=$DebugFlag "
             InsertLog "allchannels=$allchannels; allmodules=$allmodules"
             InsertLog "ModuleReg=$ModuleRegItem"
             InsertLog "ChannelReg=$ChRegItem"
             InsertLog "Rmsg=$Rmsg"
             InsertLog "Wmsg=$Wmsg"
           InsertLog "HasBeenConfigured = [::VMEConfig::ReadRegister HasBeenConfigured]"
           InsertLog "HasBeenRestored = [::VMEConfig::ReadRegister HasBeenRestored]"
            InsertLog "SoftTrigRun=$SoftTrigRun"
   }

   proc do_Select {i} {    ;# select module
        variable ModuleIndex

         set ModuleIndex $i
         UpdateModule
   }

   proc UpdateModule {} {

       do_UpdateModuleRegisters
       do_UpdateChannelRegisters
   }

   proc UpdateChannel {} {

       do_UpdateChannelRegisters
   }

   proc do_UpdateModuleRegisters {} {
        variable ModuleRegMenu

        foreach item $ModuleRegMenu  {
            set updateproc [lindex $item 3]
            if {$updateproc != "null"} {eval $updateproc}
        }
   }

   proc do_UpdateChannelRegisters {} {
       variable ChRegMenu
       foreach item $ChRegMenu  {
           set updateproc [lindex $item 3]
           if {$updateproc != "null"} {eval $updateproc}
       }
   }

   proc do_SelectModuleRegister {w} {
        variable ModuleReg
        variable ModuleRegItem
        variable ModuleRegMenu

      foreach item $ModuleRegMenu  {
         set name [lindex $item 0]
         if {"$w" == "$name"} {
             set ModuleRegItem $item
             set ModuleReg [lindex $item 1]
         }
      }
   }

   proc do_SelectChannelRegister {w} {
        variable ChReg
        variable ChRegItem
        variable ChRegMenu

      foreach item $ChRegMenu  {
         set name [lindex $item 0]
         if {"$w" == "$name"} {
             set ChRegItem $item
             set ChReg [lindex $item 1]
         }
      }
   }

    proc do_SetModuleRegister {w v} {
        variable DebugFlag
        variable $w

        if {$DebugFlag == 1} {InsertLog "do_SetModuleRegister: $w = $v"}

        switch -glob $w {
            Config.*        {set $w $v; SetConfig}
            BuffOrg         {set $w $v; SetBuffOrg}
            GrpEnableMask*  {set $w $v; SetGrpEnable}
            FPTrigEnable*   {set $w $v; SetFPTrigEnable}
            TrigSrcEnable*  {set $w $v; SetTrigSrcEnable}
            TrigCoincLevel  {set $w $v; SetTrigSrcEnable}
            FPIO.B*         {set $w $v; SetFPIO}
            FPIOData        {set $w $v; SetFPIOData}
            CustSize        {set $w $v; SetCustSize}
            PostTrig        {set $w $v; SetPostTrig}
            MonitorMode     {set $w $v; SetMonitorMode}
            MonitorDAC      {set $w $v; SetMonitorDAC}

            ConfigReload    -
            Reset           {WriteRegister $w 0; after 1000}

            AcqControl  -
            Clear -
            BLTEventNum -
            BufferOrg -
            SoftTrig        {WriteRegister $w $v}

            AcqCtrl*        {set $w $v; SetAcqCtrl}
            AcqStatus*      {}
            VMECtrl*        {set $w $v; SetVMECtrl}
            VMEStatus*      {}

            default {set $w $v}
        }
   }

    proc do_SetChannelRegister {w i vl} {
        variable allchannels
        variable DebugFlag

        if {$DebugFlag == 1} {InsertLog "do_SetChannelRegister:  $w $i = $vl"}

        for {set j 0} {$j < 8} {incr j} {
           if {$allchannels == 0 && $j != $i} {continue}
           variable Grp$j.$w; set Grp$j.$w $vl
        }

        if {$allchannels == 0} {WriteRegister Grp$i.$w $vl} else {WriteRegister $w $vl}
   }

   proc do_SoftTrig {n} {
      variable SoftTrigRun

      switch $n {
        1   {WriteRegister SoftTrig 1}
        0     {puts "V1740 finished SoftTrig run"; set SoftTrigRun 0}
        100   {set SoftTrigRun 1; after 100 ::V1740::do_SoftTrig 99}
        99999 {puts "V1740 starting SoftTrig run"; set SoftTrigRun 1; after 100 ::V1740::do_SoftTrig 99998}
        99998 {if {$::V1740::SoftTrigRun ==0} return; WriteRegister SoftTrig 1; after 100 ::V1740::do_SoftTrig 99998}
        default  {if {$::V1740::SoftTrigRun ==0} return; WriteRegister SoftTrig 1; if {$n > 0} {after 100 ::V1740::do_SoftTrig [expr $n-1]}}
      }
   }


    proc do_Save {which} {
        variable ModuleIndex
        variable ModuleName
        variable ModuleRegMenu
        variable ChRegMenu
        variable SaveRestoreMenu

        set SAV $ModuleIndex

           set ModuleIndex $which
           InsertLog "Saving Settings for CAEN [set ModuleName]#[set ModuleIndex]"

           foreach item $ModuleRegMenu  {
               set saveproc [lindex $item 5]
               if {$saveproc != "null"} {eval $saveproc}
           }

           foreach item $ChRegMenu  {
               set saveproc [lindex $item 5]
               if {$saveproc != "null"} {eval $saveproc}
           }

           foreach item $SaveRestoreMenu {

               DBSetValue $item  [ReadRegister $item]
           }
 

        set ModuleIndex $SAV
    }

    proc do_SaveAll {} {
        variable ModuleName
        variable ModuleMenu

        foreach module $ModuleMenu {do_Save $module}

        InsertLog "Settings for All CAEN [set ModuleName] modules updated in DB"

        Elog INFORMATION "Settings for All CAEN [set ModuleName] modules updated in DB"
    }

    proc do_Restore {which} {
        variable ModuleIndex
        variable ModuleName
        variable ModuleMenu
        variable ModuleRegMenu
        variable ChRegMenu
        variable SaveRestoreMenu
        variable allmodules
        variable allchannels

        set SAV1 $ModuleIndex
        set SAV2 $allmodules
        set SAV3 $allchannels

        set allmodules 0
        set allchannels 0
        set ModuleIndex $which

        InsertLog "Restoring Settings for CAEN [set ModuleName]#[set ModuleIndex]"

#        WriteRegister Reset 0   ;# software reset modules

           foreach item $ModuleRegMenu  {
               set restproc [lindex $item 6]
               if {$restproc != "null"} {eval $restproc}
           }

           foreach item $ChRegMenu  {
               set restproc [lindex $item 6]
               if {$restproc != "null"} {eval $restproc}
           }

           foreach item $SaveRestoreMenu {

               WriteRegister $item [DBGetValue $item]
           }

        set ModuleIndex $SAV1

        set allmodules $SAV2
        set allchannels $SAV3

    }

    proc do_RestoreAll {} {
        variable ModuleNameDigitalGain
        variable ModuleMenu

       foreach module $ModuleMenu {do_Restore $module}

        ::VMEConfig::WriteRegister HasBeenRestored 1

        InsertLog "Settings for All CAEN [set ModuleName] modules restored from DB"

        Elog INFORMATION "Settings for All CAEN [set ModuleName] modules restored from DB"
    }

    proc ReadDataFIFO {} {

        set datafifo [ReadRegister Output_Buffer]
        InsertLog "read data fifo returned: rc = [first $datafifo]; count=[second $datafifo]"

    }

     proc do_click {} {
       variable cgidata
       variable started
       variable LogFlag
       variable DebugFlag
       variable ShowLogFlag
       variable ModuleIndex

       set cgidata [ncgi::nvlist]

       if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}

       if {[llength $cgidata] == 0} then {return}

       if {[llength $cgidata] < 2} then {
           InsertLog "CGI data is too short: $cgidata"
       } else {
           if {[lindex $cgidata 0] != "Widget"} then {
               InsertLog "Invalid CGI data: $cgidata"
           } else {
               set w [lindex $cgidata 1]

               switch -glob [lindex $w 0] {

                   RESET    {set started 0}
                   CLEAR    {EmptyLog}
                   ELOG     {PrintLog}
                   RELOAD   {UpdateModule}
                   VARIABLES {show_variables}

                   HIDE     {set ShowLogFlag 0}
                   SHOW     {set ShowLogFlag 1}
                   LOGON    {set LogFlag 1}
                   LOGOFF   {set LogFlag 0}
                   DEBUGON    {set DebugFlag 1}
                   DEBUGOFF   {set DebugFlag 0}

                   ALLCHANS   {variable allchannels; set allchannels [expr ($allchannels + 1) & 1]}
                   ALLMODULES {variable allmodules; set allmodules [expr ($allmodules + 1) & 1]}

                   MODULE   {do_Select [lindex $w 1]}

                   MREGISTER      {do_SelectModuleRegister [string range $w 10 end]}
                   MREGISTERHELP  {do_RegisterHelp Module}

                   EXPERT {
                      switch [lindex $w 1] {
                        Help      {do_ExpertHelp}
                        Reset -
                        Clear -
                        ConfigReload  {do_SetModuleRegister [lindex $w 1] 0}
                        SoftTrig  {do_SoftTrig [lindex $w 2]}
                        AcqStart  {source [file join V1740 AcqStart.tcl]}
                        AcqHalt   {source [file join V1740 AcqHalt.tcl]}
                        DataFIFO    {ReadDataFIFO}
                        Scratch     {WriteRegister Scratch 12345}
                        default   {}
                      }
                   }
                   EXPERTHELP    {do_ExpertHelp}

                   CHREGISTER      {do_SelectChannelRegister [string range $w 11 end]}
                   CHREGISTERHELP  {do_RegisterHelp Ch}

                   MSETTING      {do_SetModuleRegister [lindex $w 1] [lindex $w 2]}
                   MSETTINGS     {do_SetModuleRegister [lindex $cgidata 4] [lindex $cgidata 5]}

                   CHSETTING     {do_SetChannelRegister [lindex $w 1] [lindex $w 2] [lindex $w 3]}

                   PLLWRITEFLASH {PLLWriteFlash}
                   PLLREADFLASH  {PLLReadFlash}
                   PLLUPDATEFLASH {PLLUpdateFlash}
                   PLLRELOADFLASH {PLLReloadFlash}

                   PLLCLKENABLE {
                       switch [lindex $w 1] {
                           false {variable ClockDelayEnable; set ClockDelayEnable 0}
                           true  {variable ClockDelayEnable; set ClockDelayEnable 1}
                       }
                       PLLUpdateFlash
                   }

                   PLLOUTENABLE {
                       switch [lindex $w 1] {
                           false {variable OutputClockEnable; set OutputClockEnable 0}
                           true  {variable OutputClockEnable; set OutputClockEnable 1}
                       }
                       PLLUpdateFlash
                   }

                   PLLCLKDELAY   {variable ClockDelay; set ClockDelay [lindex $w 1]; PLLUpdateFlash}
                   
                   SAVEALL      {do_SaveAll}
                   RESTALL      {do_RestoreAll}
                   SAVE         {do_Save $ModuleIndex}
                   RESTORE      {do_Restore $ModuleIndex}

                   REDISPLAY  {} 

                   MainLog {}
                   undefined {}
                   default {InsertLog "unexpected CGI data: [lindex $w 0]: $cgidata"}
               }
           }
       }
       UpdateModule
    }

}   ;# end of if {$started == 0}


     if {$LogFlag == 1} {InsertLog "Started: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}

